/*
 * YouGrabber
 *
 * A lightweight, command line YouTube.com video downloader, made from scratch
 * in ANSI C.
 *
 * Copyright (C) 2006-2008 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "include/common.h"

static const unsigned __FILE_MAX = 999;
static const char __CHAR_VALID[] = "()[]._-";
static const char __CHAR_SPACE = '_';

/* print a string with the yg info header */
void yg_print(char *str)
{
  fprintf(stderr, "%s v%s - Copyright (C) %s <%s>\n\n%s\n\n",
    YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL, str);
}

/* configure an easy cURL handle */
CURL *yg_handle_config(CURL *handle, YGC *c_ptr, FILE *stream, const char *url,
  char *error_buf, int p_use, void *p_func, void *p_data, char *post)
{
  curl_easy_setopt(handle, CURLOPT_ERRORBUFFER, error_buf);
  curl_easy_setopt(handle, CURLOPT_URL, url);
  curl_easy_setopt(handle, CURLOPT_WRITEDATA, stream);
  curl_easy_setopt(handle, CURLOPT_USERAGENT, yg_config_agent(c_ptr));
  curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1);
  curl_easy_setopt(handle, CURLOPT_NOPROGRESS, !p_use);
  curl_easy_setopt(handle, CURLOPT_PROGRESSDATA, p_data);
  curl_easy_setopt(handle, CURLOPT_PROGRESSFUNCTION, p_func);

  curl_easy_setopt(handle, CURLOPT_COOKIEFILE, yg_config_cookie(c_ptr));
  curl_easy_setopt(handle, CURLOPT_COOKIEJAR, yg_config_cookie(c_ptr));

  curl_easy_setopt(handle, CURLOPT_PROXY, yg_config_proxy_host(c_ptr));
  curl_easy_setopt(handle, CURLOPT_PROXYPORT, yg_config_proxy_port(c_ptr));
  curl_easy_setopt(handle, CURLOPT_PROXYTYPE, yg_config_proxy_type(c_ptr));
  curl_easy_setopt(handle, CURLOPT_PROXYAUTH, CURLAUTH_ANYSAFE);
  curl_easy_setopt(handle, CURLOPT_PROXYUSERPWD,
    yg_config_proxy_credentials(c_ptr));

  if (post)
  {
    curl_easy_setopt(handle, CURLOPT_POST, 1);
    curl_easy_setopt(handle, CURLOPT_POSTFIELDS, post);
  }

  return handle;
}

/* cut a string between two delimiters (returns new alloc'd string) */
char *yg_strcut(const char *str, const char *d1, const char *d2)
{
  unsigned short start_pos = 0;
  unsigned short start_set = 0;
  unsigned short stop_pos = 0;
  unsigned short stop_set = 0;
  unsigned i;
  unsigned j;

  for (i = j = 0; i < strlen(str); ++i)
  {
    if (!start_set)
    {
      j = (str[i] == d1[j]) ? j+1 : 0;

      if (strlen(d1) == j || strlen(d1) == 0)
      {
        start_pos = i;
        start_set = 1;
        j = 0;
      }
    }

    if (start_set)
    {
      j = (str[i] == d2[j]) ? j+1 : 0;

      if (strlen(d2) == 0 || strlen(d2) == j)
      {
      	/* if the second delimiter is empty, return the string length */
        stop_pos = (strlen(d2) == 0) ? strlen(str) : (i+1)-strlen(d2);
        stop_set = 1;
      }
    }

    if (start_set && stop_set)
      break;
  }

  /* return original string if one of the delimiters doesn't match */
  if (!start_set || !stop_set)
    return g_strdup(str);

  char buf[strlen(str)];

  for (i = start_pos+1, j = 0; i < stop_pos; ++i, ++j)
    buf[j] = str[i];

  buf[j] = 0;

  return g_strdup(buf);
}

/* generate a file name with full path (returns new alloc'd string) */
char *yg_filegen(const char *path, const char *name, const char *ext)
{
  char sn[strlen(name)]; /* sanitized name */
  char *file = NULL;
  unsigned i;
  unsigned p;

  for (i = p = 0; i < strlen(name); ++i)
    sn[p++] = (isalnum(name[i]) || strchr(__CHAR_VALID, name[i]))
      ? name[i] : __CHAR_SPACE;

  sn[p] = 0;

  for (i = 0; i <= __FILE_MAX; ++i)
  {
    if (i)
    {
    	free(file);
    	file = g_strdup_printf("%s/%s_%.3d.%s", path, sn, i, ext);
    }
    else
      file = g_strdup_printf("%s/%s.%s", path, sn, ext);

    if (access(file, F_OK) != 0)
      return file;
  }

  free(file);
  return file = NULL;
}

/* convert a UTF8 string to the current locale (returns new alloc'd string) */
char *yg_utf2locale(char *str)
{
  char *lc_str = NULL;
	size_t bytes_read;
	size_t bytes_written;

  /* locale converted string */
  lc_str = g_locale_from_utf8(str, -1, &bytes_read, &bytes_written, NULL);

  /* return original string if conversion fails */
  return (lc_str != NULL) ? lc_str : g_strdup(str);
}
