/*
 * YouGrabber
 *
 * A lightweight, command line YouTube.com video downloader, made from scratch
 * in ANSI C.
 *
 * Copyright (C) 2006-2008 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "include/common.h"

static const char __FILE_TEMP[] = {"VIDEODATA"};
static const char __EXT_TEMP[] = {"part"};
static const char __EXT_VIDEO[] = {"flv"};

/* match strings and delimiters */
static const char __FLAGGED_URL_MATCH[] = {"verify_age?"};

static const char __ERROR_MATCH[] = {"class=\"errorBox\""};

static const char __VIDEO_ID_DELIM_1[] = {"?v="};
static const char __VIDEO_ID_DELIM_2[] = {""};

static const char __TITLE_MATCH[] = {"<meta name=\"title\""};
static const char __TITLE_DELIM_1[] = {"content=\""};
static const char __TITLE_DELIM_2[] = {"\">"};

static const char __URL_T_MATCH[] = {"swfArgs"};
static const char __URL_T_DELIM_1[] = {"t\": \""};
static const char __URL_T_DELIM_2[] = {"\","};

static const char __YOUTUBE_URL_VIDEO[] =
  {"http://www.youtube.com/get_video?video_id="};

struct __yg_download
{
  char curl_error[CURL_ERROR_SIZE];
  unsigned short status;
  CURL *handle;
  FILE *stream;
  double time;
  double speed;
  YGC *c_ptr;

  struct
  {
    char *name;
    double size_now;
    double size_total;
  } file;

  struct
  {
    char *title;
    char *url;
    char *id;
  } video;

  pthread_t thread;
};

/* update download stats (cURL progress callback function) */
static int __yg_download_update(void *clientp, double dltotal, double dlnow,
  double ultotal, double ulnow)
{
  YGD *d_ptr = NULL;
  char *url = NULL;

  (void) ultotal;
  (void) ulnow;

  d_ptr = (YGD*) clientp;

  d_ptr->file.size_now = dlnow;
  d_ptr->file.size_total = dltotal;

  curl_easy_getinfo(d_ptr->handle, CURLINFO_EFFECTIVE_URL, &url);

  if (g_ascii_strcasecmp(url, d_ptr->video.url) != 0)
  {
    free(d_ptr->video.url);
    d_ptr->video.url = g_strdup(url);
  }

  curl_easy_getinfo(d_ptr->handle, CURLINFO_TOTAL_TIME, &d_ptr->time);
  curl_easy_getinfo(d_ptr->handle, CURLINFO_SPEED_DOWNLOAD, &d_ptr->speed);

  return 0;
}

/* do the video download */
static int __yg_download_perform(YGD *d_ptr)
{
  unsigned short int got_title = 0;
  unsigned short int got_url = 0;
  char buf[256];

  /* wait for the login result */
  while (yg_config_login_status(d_ptr->c_ptr) == YG_LOGIN_OK_CONNECTING)
    sleep(2);

  if (yg_config_login_status(d_ptr->c_ptr) == YG_LOGIN_ERROR_CONNECT)
    return d_ptr->status = YG_DOWNLOAD_ERROR_CANCELED;
  else
    d_ptr->status = YG_DOWNLOAD_OK_DOWNLOADING;

  if (!(d_ptr->file.name = yg_filegen(yg_config_download_dir(d_ptr->c_ptr),
    __FILE_TEMP, __EXT_TEMP)))
    return d_ptr->status = YG_DOWNLOAD_ERROR_FILE_LIMIT;

  if (!(d_ptr->stream = fopen(d_ptr->file.name, "w+")))
    return d_ptr->status = YG_DOWNLOAD_ERROR_FILE_LIMIT;

  /* download phase 1 */
  d_ptr->handle = yg_handle_config(d_ptr->handle, d_ptr->c_ptr, d_ptr->stream,
    d_ptr->video.url, d_ptr->curl_error, 1, __yg_download_update, d_ptr, NULL);

  if (curl_easy_perform(d_ptr->handle) != CURLE_OK)
    return d_ptr->status = YG_DOWNLOAD_ERROR_CURL;

  /* flagged video? only confirm birth date if the login was successful */
  if (strstr(d_ptr->video.url, __FLAGGED_URL_MATCH)
    && yg_config_login_status(d_ptr->c_ptr) == YG_LOGIN_OK_LOGGED)
  {
    g_snprintf(buf, sizeof(buf),
      "next_url=/watch?v=%s&action_confirm=Confirm+Birth+Date",
      d_ptr->video.id);

    d_ptr->handle = yg_handle_config(d_ptr->handle, d_ptr->c_ptr,
      d_ptr->stream, d_ptr->video.url, d_ptr->curl_error, 1,
      __yg_download_update, d_ptr, buf);

    if (curl_easy_perform(d_ptr->handle) != CURLE_OK)
      return d_ptr->status = YG_DOWNLOAD_ERROR_CURL;
  }

  /* download phase 2 */
  fflush(d_ptr->stream);
  rewind(d_ptr->stream);

  while (fgets(buf, sizeof(buf), d_ptr->stream) || (!got_title && !got_url)
    || !strstr(buf, __ERROR_MATCH))
  {
    if (feof(d_ptr->stream))
      break;

    /* video title */
    if (strstr(buf, __TITLE_MATCH))
    {
    	char *title = NULL;

      title = yg_strcut(buf, __TITLE_DELIM_1, __TITLE_DELIM_2);

      free(d_ptr->video.title);
      d_ptr->video.title = yg_utf2locale(title);

      free(title);
      got_title = 1;
    }

    /* URL "t" parameter */
    if (strstr(buf, __URL_T_MATCH))
    {
      char *t = NULL;

      t = yg_strcut(buf, __URL_T_DELIM_1, __URL_T_DELIM_2);

      free(d_ptr->video.url);
      d_ptr->video.url = g_strdup_printf("%s%s&t=%s", __YOUTUBE_URL_VIDEO,
        d_ptr->video.id, t);

      free(t);
      got_url = 1;
    }
  }

  /* download phase 3 */
  if (got_title && got_url)
  {
    fflush(d_ptr->stream);
    rewind(d_ptr->stream);

    d_ptr->handle = yg_handle_config(d_ptr->handle, d_ptr->c_ptr,
      d_ptr->stream, d_ptr->video.url, d_ptr->curl_error, 1,
      __yg_download_update, d_ptr, NULL);

    if (curl_easy_perform(d_ptr->handle) != CURLE_OK)
      return d_ptr->status = YG_DOWNLOAD_ERROR_CURL;
  }
  else
    return d_ptr->status = YG_DOWNLOAD_ERROR_VIDEO_UNAVAILABLE;

  /* download phase 4 */
  fflush(d_ptr->stream);

  g_strlcpy(buf, d_ptr->file.name, sizeof(buf));

  free(d_ptr->file.name);

  d_ptr->file.name = yg_filegen(yg_config_download_dir(d_ptr->c_ptr),
    d_ptr->video.title, __EXT_VIDEO);

  rename(buf, d_ptr->file.name);

  return d_ptr->status = YG_DOWNLOAD_OK_FINISHED;
}

/* create a new download and return pointer */
YGD *yg_download_new(YGC *c_ptr, char *url)
{
  YGD *d_ptr = NULL;

  if (!(d_ptr = (YGD *) malloc(sizeof(YGD))))
    return NULL;

  if (!(d_ptr->handle = curl_easy_init()))
  {
    free(d_ptr);

    return NULL;
  }

  d_ptr->c_ptr = c_ptr;
  d_ptr->status = YG_DOWNLOAD_OK_WAITING;
  d_ptr->time = 0.0;
  d_ptr->speed = 0.0;
  d_ptr->file.name = NULL;
  d_ptr->file.size_now = 0.0;
  d_ptr->file.size_total = 0.0;
  d_ptr->video.title = yg_utf2locale(url);
  d_ptr->video.url = g_strdup(url);
  d_ptr->video.id = yg_strcut(url, __VIDEO_ID_DELIM_1, __VIDEO_ID_DELIM_2);

  if (pthread_create(&d_ptr->thread, NULL, (void *) __yg_download_perform,
    d_ptr))
  {
    yg_download_free(d_ptr, NULL);

    return NULL;
  }

  return d_ptr;
}

/* free download resources (GFunc() type function) */
void yg_download_free(gpointer data, gpointer user_data)
{
  unsigned file_close[] = {0, 0, 1, 1, 0, 0, 1};
  unsigned file_unlink[] = {0, 0, 1, 1, 0, 0, 0};
  YGD *d_ptr = NULL;

  (void) user_data;

  d_ptr = (YGD*) data;

  pthread_join(d_ptr->thread, NULL);

  if (file_close[d_ptr->status])
    fclose(d_ptr->stream);

  if (file_unlink[d_ptr->status])
    unlink(d_ptr->file.name);

  curl_easy_cleanup(d_ptr->handle);
  free(d_ptr->file.name);
  free(d_ptr->video.title);
  free(d_ptr->video.url);
  free(d_ptr->video.id);
  free(d_ptr);
}

/* get the download status */
int yg_download_status(YGD *d_ptr)
{
  return d_ptr->status;
}

/* get the download curl error string */
char *yg_download_curl_error(YGD *d_ptr)
{
  return d_ptr->curl_error;
}

/* get the download speed */
double yg_download_speed(YGD *d_ptr)
{
  return d_ptr->speed;
}

/* get the download elapsed time */
double yg_download_time(YGD *d_ptr)
{
  return d_ptr->time;
}

/* get the download file name */
char *yg_download_file_name(YGD *d_ptr)
{
  return d_ptr->file.name;
}

/* get the download current file size */
double yg_download_file_size_now(YGD *d_ptr)
{
  return d_ptr->file.size_now;
}

/* get the download total file size */
double yg_download_file_size_total(YGD *d_ptr)
{
  return d_ptr->file.size_total;
}

/* get the download video title */
const char *yg_download_video_title(YGD *d_ptr)
{
  return d_ptr->video.title;
}

/* get the download video URL */
char *yg_download_video_url(YGD *d_ptr)
{
  return d_ptr->video.url;
}
