/*
 * YouGrabber
 *
 * A lightweight, command line YouTube.com video downloader, made from scratch
 * in ANSI C.
 *
 * Copyright (C) 2006-2008 Quetzy Garcia, <quetzyg@users.sourceforge.net>
 * For news and updates, see <http://yougrabber.sourceforge.net/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "include/common.h"

static unsigned __ACTIVE_DOWNLOADS = 0;

static const double __MEGABYTE = 1024.0*1024.0;
static const double __KILOBYTE = 1024.0;
static const double __MINUTE = 59.0;
static const double __PERCENT = 100.0;

/* interface current and minimum sizes */
static unsigned __LINES = 0;
static unsigned __COLS = 0;
static const unsigned short __MIN_LINES = 20;
static const unsigned short __MIN_COLS = 80;

struct __yg_interface
{
  YGC *c_ptr;
  GList *d_list;
  WINDOW *window;
  YGD *cur_dl;
};

/* start ncurses */
static void __yg_interface_ncstart(void)
{
  initscr();
  cbreak();
  noecho();
  curs_set(0);
  halfdelay(1);
  keypad(stdscr, TRUE);
}

/* stop ncurses */
static void __yg_interface_ncstop(void)
{
  clear();
  refresh();
  endwin();
}

/* set the interface size */
static void __yg_interface_resize(YGI *i_ptr)
{
  getmaxyx(i_ptr->window, __LINES, __COLS);

  __LINES = (__LINES < __MIN_LINES) ? __MIN_LINES : __LINES;
  __COLS = (__COLS < __MIN_COLS) ? __MIN_COLS : __COLS;
}

/* set the number of active downloads */
static void __yg_interface_active_downloads(YGI *i_ptr)
{
  unsigned status_download[] = {0, 0, 0, 0, 1, 1, 0};
  unsigned i = 0;
  GList *node = NULL;

  for (node = g_list_first(i_ptr->d_list); node; node = node->next)
    i += status_download[yg_download_status((YGD*) node->data)];

  __ACTIVE_DOWNLOADS = i;
}

/* update the interface */
static void __yg_interface_refresh(YGI *i_ptr)
{
  char *config_file = NULL;
  char *proxy_host = NULL;
  char *url = NULL;
  double dlnow = 0.0;
  double dltotal = 0.01; /* avoid divisions by zero */
  double time = 0.0;

  char *status_youtube[] =
  {
    "Unable to connect (network problem?)",
    "Failed (flagged videos will be unavailable)",
    "Skipped (flagged videos will be unavailable)",
    "Connecting...",
    "Logged"
  };

  char *status_download[] =
  {
    "Download canceled",
    "File name error (limit reached)",
    yg_download_curl_error(i_ptr->cur_dl),
    "Video unavailable",
    "Waiting...",
    "Downloading video data...",
    "Download complete"
  };

  config_file = yg_config_file(i_ptr->c_ptr);
  proxy_host = yg_config_proxy_host(i_ptr->c_ptr);
  dlnow = yg_download_file_size_now(i_ptr->cur_dl);
  dltotal += yg_download_file_size_total(i_ptr->cur_dl);
  time = yg_download_time(i_ptr->cur_dl);
  url = yg_download_video_url(i_ptr->cur_dl);

  /* general info */
  werase(i_ptr->window);

  mvwprintw(i_ptr->window, 0, 0, "%s v%s - Copyright (C) %s <%s>",
    YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL);

  mvwhline(i_ptr->window, 1, 0, ACS_HLINE, __COLS);

  mvwprintw(i_ptr->window, 3, 1, "YOUTUBE LOGIN STATUS: %s",
    status_youtube[yg_config_login_status(i_ptr->c_ptr)]);

  mvwprintw(i_ptr->window, 4, 1,  "PROXY: %s",
    (proxy_host) ? proxy_host : "None");

  mvwprintw(i_ptr->window, 5, 1,
    (config_file) ? "SETTINGS: '%s'" : "SETTINGS: %s",
    (config_file) ? config_file : "Default");

  mvwprintw(i_ptr->window, 6, 1, "DOWNLOAD DIR: '%s'",
    yg_config_download_dir(i_ptr->c_ptr));

  /* download info */
  mvwhline(i_ptr->window, __LINES-11, 0, ACS_HLINE, __COLS);

  mvwprintw(i_ptr->window, __LINES-11, 2,
    " DOWNLOAD %.3d/%.3d [%.3d REMAINING] ",
    g_list_index(i_ptr->d_list, i_ptr->cur_dl)+1,
    g_list_length(i_ptr->d_list), __ACTIVE_DOWNLOADS);

  mvwprintw(i_ptr->window, __LINES-10, 0, "TITLE: %s",
    yg_download_video_title(i_ptr->cur_dl));

  mvwprintw(i_ptr->window, __LINES-9, 0, "FILE: %s",
    basename(yg_download_file_name(i_ptr->cur_dl)));

  mvwprintw(i_ptr->window, __LINES-8, 0, "STATUS: %s",
    status_download[yg_download_status(i_ptr->cur_dl)]);

  mvwprintw(i_ptr->window, __LINES-7, 0,
    "DOWNLOADED: %.2f%% [%.2f of %.2f MB]\n",
    (dltotal < dlnow) ? 100.0 : (dlnow*__PERCENT)/dltotal,
    dlnow/__MEGABYTE, dltotal/__MEGABYTE);

  mvwprintw(i_ptr->window, __LINES-6, 0, "SPEED: %.2f KB/sec",
    yg_download_speed(i_ptr->cur_dl)/__KILOBYTE);

  mvwprintw(i_ptr->window, __LINES-5, 0, "ELAPSED: %d min %.f sec",
    (int) (time/__MINUTE), fmod(time, __MINUTE));

  mvwprintw(i_ptr->window, __LINES-4, 0,
    (strlen(url) > 128) ? "URL: %.128s (...)" : "URL: %s", url);

  /* footer */
  mvwhline(i_ptr->window, __LINES-2, 0, ACS_HLINE, __COLS);

  mvwprintw(i_ptr->window, __LINES-1, 0,
    "LEFT/RIGHT ARROW KEYS: browse between downloads - %s",
    (__ACTIVE_DOWNLOADS > 0) ?
    "Wait for downloads to finish" : "Press Q to quit");

  wrefresh(i_ptr->window);
}

/* create interface and return pointer */
YGI *yg_interface_new(YGC *c_ptr, GList *d_list)
{
  YGI *i_ptr;

  i_ptr = NULL;

  if (!(i_ptr = (YGI*) malloc(sizeof(YGI))))
    return NULL;

  __yg_interface_ncstart();

  i_ptr->c_ptr = c_ptr;
  i_ptr->d_list = d_list;
  i_ptr->window = stdscr; /* use default window */
  i_ptr->cur_dl = NULL;

  return i_ptr;
}

/* free interface resources */
void yg_interface_free(YGI *i_ptr)
{
  __yg_interface_ncstop();

  g_list_foreach(i_ptr->d_list, yg_download_free, NULL);
  g_list_free(i_ptr->d_list);

  yg_config_free(i_ptr->c_ptr);

  free(i_ptr);
}

/* interface event processing loop */
void yg_interface_main_loop(YGI *i_ptr)
{
  unsigned short quit = 0;
  GList *node_current = NULL;
  GList *node_next = NULL;

  node_current = g_list_first(i_ptr->d_list);

  __yg_interface_resize(i_ptr);

  while(!quit)
  {
    node_current = (!node_next) ? node_current : node_next;
    i_ptr->cur_dl = node_current->data;

    __yg_interface_active_downloads(i_ptr);
    __yg_interface_refresh(i_ptr);

    switch (getch())
    {
      case KEY_RESIZE:
        __yg_interface_ncstop();
        __yg_interface_ncstart();
        __yg_interface_resize(i_ptr);
      break;

      case KEY_LEFT:
        node_next = g_list_previous(node_current);
      break;

      case KEY_RIGHT:
        node_next = g_list_next(node_current);
      break;

      /* ESC, Q or q */
      case 27:
      case 81:
      case 113:
        quit = !__ACTIVE_DOWNLOADS;
      break;
    }
  }
}
